#include <TinyGPS++.h> // library gps
#include <SoftwareSerial.h> // library rx tx fr use with gps
#include "DHT.h" // library dht 
////////// library for oled //////////
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
////////// library for http client method ////////// 
#include <ArduinoJson.h>
#include <Arduino_JSON.h>
#include <HTTPClient.h>
#include <WiFi.h> // library wifi

#define DHTPIN 15  // declare dht pin 15
#define DHTTYPE DHT11 // dht type 11
#define SCREEN_WIDTH 128 // OLED display width,  in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels

// declare an SSD1306 display object connected to I2C
Adafruit_SSD1306 oled(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);      

static const int RXPin = 3, TXPin = 1;
static const uint32_t GPSBaud = 9600; //buad rate of GPS
const int buzzer = 23; //buzzer to arduino pin 23
int serial_id = 3; // unique id

const char* ssid = "TrueGigatexFiber_2.4G"; // wifi name
const char* password = "Boat2543!"; // wifi password

const char* url_post = "https://polite-mole-50.loca.lt/test"; // link database for post, use for test database's connection 
const char* url_get = "https://polite-mole-50.loca.lt/userdata/serialID/3"; // get data from database, show on oled
const char* url_put = "https://polite-mole-50.loca.lt/petStatus/3"; // update data to database 

unsigned long lastTime = 0; // use for control get method delay
unsigned long timerDelay = 2000; // Set timer to 2 seconds 

////////// keep data from get method for show on oled ///////////
String sensorReadings; 
String sensorReadingsArr[4];
String petName = "";

////////// set json format //////////
char str[500]; // length of json  
const int _size = 3 * JSON_OBJECT_SIZE(3);
StaticJsonDocument<_size> JSONPost;
StaticJsonDocument<_size> JSONPut;

////////// declare variables //////////
DHT dht(DHTPIN, DHTTYPE);

TinyGPSPlus myGPS;
SoftwareSerial mySerial(RXPin, TXPin);

 
void setup(){
  Serial.begin(115200); // set baudrate serial monitor 
  mySerial.begin(GPSBaud); // rx tx with GPS
  pinMode(buzzer, OUTPUT); // Set buzzer - pin 23 as an output
  dht.begin(); // start dht
  digitalWrite(buzzer, HIGH); // stop buzzer

// check if oled error
  if (!oled.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { 
    Serial.println(F("SSD1306 allocation failed")); 
    while (true);
  }
  
  delay(2000);         // wait for initializing
  oled.clearDisplay(); // clear display

  oled.setTextSize(2);          // text size
  oled.setTextColor(WHITE);     // text color
  oled.setCursor(0, 10);        // position to display
  oled.println("SAWATECH!"); // text to display
  oled.display();               // show on OLED
  
  WiFi_Connect(); // connect wifi
}

 
void loop()
{
  float h = dht.readHumidity();
  float t = dht.readTemperature();

//check if dht error or can't read temp or humid
  if (isnan(h) || isnan(t)) {
    Serial.println(F("Failed to read from DHT sensor!"));
    return;
  }
  
  while (mySerial.available() > 0) // check GPS work?
    if (myGPS.encode(mySerial.read())){ // check GPS read?
      GPSinfo(); // GPS function
      Serial.print(F("Humidity: ")); 
      Serial.print(h);
      Serial.print(F("%  Temperature: "));
      Serial.print(t);
      Serial.println(F("°C "));
      if (t > 30){
        Serial.println(F("ALARMMMMMMMMMMMMM"));
        digitalWrite(buzzer, LOW);
        delay(2000);      
        _get();
        //_post(serial_id,h,t,myGPS.location.lat(),myGPS.location.lng());
        _put(serial_id,petName,h,t,myGPS.location.lat(),myGPS.location.lng());
      }
      else{
       digitalWrite(buzzer, HIGH);
       _get();
       //_post(serial_id,h,t,myGPS.location.lat(),myGPS.location.lng());
       _put(serial_id,petName,h,t,myGPS.location.lat(),myGPS.location.lng());
      }
    }

// check if GPS can't work for 5 secs
  if (millis() > 5000 && myGPS.charsProcessed() < 10) {
    Serial.println("No GPS detected: check wiring.");
    while(true);
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////

void WiFi_Connect() {
  WiFi.disconnect();
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED){
    delay(1000);
    Serial.println("Connecting to WiFi...");
  }

  Serial.println("Connected to the WiFi network");  
}

void _post(int serial_id, float humid, float temp, float latitude, float longtitude) {
  if(WiFi.status() == WL_CONNECTED){
    HTTPClient http;

    http.begin(url_post);
    http.addHeader("Content-Type", "application/json");

    JSONPost["SerialID"] = serial_id;
    JSONPost["humid"] = humid;
    JSONPost["temp"] = temp;    
    JSONPost["lat"] = latitude;
    JSONPost["long"] = longtitude;
    serializeJson(JSONPost, str);
    int httpCode = http.POST(str);

     Serial.println(str);
     Serial.println(httpCode);   

    if(httpCode == HTTP_CODE_OK) {
      String payload  = http.getString();
      Serial.println(httpCode);
      Serial.println(payload);      
    }
    else{
      Serial.println(httpCode);
      Serial.println("Error on HTTP Request");      
    }
  }
  else{
    WiFi_Connect();
  }
  delay(2000);  
}

void _get() {
  if ((millis() - lastTime) > timerDelay) {
    if(WiFi.status()== WL_CONNECTED){
      sensorReadings = httpGETRequest(url_get);
      Serial.println(sensorReadings);
      JSONVar myObject = JSON.parse(sensorReadings);
  
      if (JSON.typeof(myObject) == "undefined") {
        Serial.println("Parsing input failed!");
        return;
      }
    
      Serial.print("JSON object = ");
      Serial.println(myObject);
   
      JSONVar keys = myObject.keys();
    
      for (int i = 0; i < keys.length(); i++) {
        JSONVar value = myObject[keys[i]];
        Serial.print(keys[i]);
        Serial.print(" = ");
        Serial.println(value);
        sensorReadingsArr[i] = (value);
      }
      
      oled.clearDisplay(); // clear display
      
      oled.setTextSize(1);          // text size
      oled.setTextColor(WHITE);     // text color
      oled.setCursor(0, 10);        // position to display

      oled.print("name : ");
      oled.println(sensorReadingsArr[0]);
      oled.print("petName : ");
      oled.println(sensorReadingsArr[1]);
      oled.print("serialID : ");
      oled.println(sensorReadingsArr[2]);
      oled.print("tel : ");
      oled.println(sensorReadingsArr[3]);
      oled.display();

      petName = sensorReadingsArr[1];
    }
    else {
      Serial.println("WiFi Disconnected");
    }
    lastTime = millis();
  }
}

String httpGETRequest(const char* url_get) {
  HTTPClient http;
    
  http.begin(url_get);
  
  int httpResponseCode = http.GET();
  
  String payload = "{}"; 
  
  if (httpResponseCode>0) {
    Serial.print("HTTP Response code: ");
    Serial.println(httpResponseCode);
    payload = http.getString();
  }
  else {
    Serial.print("Error code: ");
    Serial.println(httpResponseCode);
  }
  http.end();

  return payload;
}

void _put(int serial_id, String petName, float humid, float temp, float latitude, float longtitude) { 
  if(WiFi.status() == WL_CONNECTED){
    HTTPClient http;

    http.begin(url_put);
    http.addHeader("Content-Type", "application/json");
    JSONPut["serialID"] = serial_id;
    JSONPut["petName"] = petName; 
    JSONPut["humid"] = humid;
    JSONPut["temp"] = temp;   
    JSONPut["lat"] = latitude;
    JSONPut["long"] = longtitude;
    serializeJson(JSONPut, str);
    int httpCode = http.PUT(str);

     Serial.println(str);
     Serial.println(httpCode);   

    if(httpCode == HTTP_CODE_OK) {
      String payload  = http.getString();
      Serial.println(httpCode);
      Serial.println(payload);      
    }
    else{
      Serial.println(httpCode);
      Serial.println("Error on HTTP Request");      
    }
  }
  else{
    WiFi_Connect();
  }
  delay(2000);  
}

void GPSinfo(){
  Serial.print("Location: "); 
  if (myGPS.location.isValid()) {
    Serial.print(myGPS.location.lat(), 6);      
    Serial.print(", ");
    Serial.print(myGPS.location.lng(), 6);     
    Serial.print("\t");                        
  } else {                                     
    Serial.print("INVALID");
  }

  Serial.print("  Date/Time: ");
  if (myGPS.date.isValid()) {
    Serial.print(myGPS.date.day());            
    Serial.print("/");
    Serial.print(myGPS.date.month());       
    Serial.print("/");
    Serial.print(myGPS.date.year());         
  } else {                                    
    Serial.print("INVALID");
  }

  Serial.print("\t");                               
  if (myGPS.time.isValid()) {
    if (myGPS.time.hour()+7 < 10) Serial.print("0");    
    Serial.print(myGPS.time.hour()+7);
    Serial.print(":");
    if (myGPS.time.minute() < 10) Serial.print("0"); 
    Serial.print(myGPS.time.minute());
    Serial.print(":");
    if (myGPS.time.second() < 10) Serial.print("0"); 
    Serial.print(myGPS.time.second());
  } else {                                  
    Serial.print("INVALID");
  }

  Serial.println();

  delay(2000);
}
